@@ -21,27 +21,69 @@ FRK <- function(f,                     # formula (compulsory)
                 BAUs = NULL,           # BAUs
                 est_error = TRUE,      # estimate measurement error
                 average_in_BAU = TRUE, # average data into BAUs
+                sum_variables = NULL,  # variables to sum rather than average
+                normalise_wts = TRUE,
                 fs_model = "ind",      # fine-scale variation component
                 vgm_model = NULL,      # variogram model for error estimation
-                K_type = "block-exponential", # type of K matrix
+                K_type = c("block-exponential", "precision", "unstructured"), # type of K matrix
                 n_EM = 100,            # max. no. of EM iterations
                 tol = 0.01,            # tolerance at which EM is assumed to have converged
-                method = "EM",         # method for parameter estimation
+                method = c("EM", "TMB"),         # method for parameter estimation
                 lambda = 0,            # regularisation parameter
                 print_lik = FALSE,     # print log-likelihood at each iteration
-                ...)                   # other arguments for BAUs/basis-function construction
+                response = c("gaussian", "poisson", "gamma",
+                             "inverse-gaussian", "negative-binomial", "binomial"), 
+                link = c("identity", "log", "square-root", "logit", "probit", "cloglog", "inverse", "inverse-squared"),
+                optimiser = nlminb,    # Optimiser for fitting (applicable only if method = 'TMB')
+                fs_by_spatial_BAU = FALSE,
+                known_sigma2fs = NULL, 
+                taper = NULL, 
+                ...)                   # other arguments for BAUs/basis-function construction, or 
 {
 
     if(!is.list(data))          # Allow for user to supply data as a single object
         data <- list(data)      # If he/she does then put it into a list
 
+    
+    ## Strings that must be lower-case (this allows users to enter 
+    ## response = "Gaussian", for example, without causing issues)
+    response  <- tolower(response)
+    link      <- tolower(link)
+    K_type    <- tolower(K_type)
+
+    ## Match arguments/function
+    K_type    <- match.arg(K_type)
+    response  <- match.arg(response)
+    link      <- match.arg(link)
+    method    <- match.arg(method)
+    optimiser <- match.fun(optimiser)
+    
+    ## FRK() is intended to be used for a novice user; simply enforce 
+    ## method = "TMB" for non-Gaussian data or when a link other than the 
+    ## identity is used, and enforce K_type = "precision" when method = "TMB"
+    if (response != "gaussian" || link != "identity") {
+        if (method != "TMB") 
+            cat("Setting method = 'TMB', as a non-Gaussian data model or a non-identity link function has been chosen.\n")
+        method <- "TMB"
+    }
+    if (method == "TMB") {
+        if (K_type != "precision") 
+            cat("For computational efficiency, setting K_type = 'precision', because method = 'TMB'; if you really want to use the K_type = 'block-exponential' with method = 'TMB', use SRE() and SRE.fit().\n")
+        K_type <- "precision"
+    }
+    
     .check_args_wrapper(f = f,              # check that the arguments are OK for SRE
                         data = data,
                         basis = basis,
                         BAUs = BAUs,
-                        est_error = est_error)
-    .check_args2(...)                      # check that the arguments are OK for SRE.fit
-    .check_args3(...)                      # check that the arguments are OK for prediction
+                        est_error = est_error, 
+                        response = response)
+    
+    # check that the arguments are OK for SRE.fit
+    .check_args2(n_EM = n_EM, tol = tol, method = method, print_lik = print_lik, 
+                 response = response, link = link, K_type = K_type, lambda = lambda,
+                 optimiser = optimiser, fs_by_spatial_BAU = fs_by_spatial_BAU, 
+                 known_sigma2fs = known_sigma2fs, BAUs = BAUs, taper = taper, ...)                      
 
     ## if there is a measurement error declared in all datasets then
     ## don't estimate it
@@ -84,18 +126,22 @@ FRK <- function(f,                     # formula (compulsory)
             max_sp_basis <- min(tot_data^(0.5),2000) # amount of basis functions to be sqrt
         } else {                                     # of data points (or 2000), else
             max_sp_basis <- 2000                     # just limit to 2000
-            if(is(manifold,"sphere")) {              # If we're on the sphere just harc
+            if(is(manifold,"sphere")) {              # If we're on the sphere just hard
                 max_basis <- NULL                    # code the default basis functions
                 nres <- 3                            # to 3 ISEA3h resolutions which is OK
                 isea3h_lo <- 2                       # in most applications
             }
         }
 
+        ## If nres is provided, don't need to use the default max number of 
+        ## basis functions. Only consider nres if method = "TMB"; we don't want 
+        ## to allow nres >= 4 if method = "EM", as it would be too slow. 
+        if(!is.null(list(...)$nres) && method == "TMB") max_sp_basis <- NULL
 
         if(!(grepl("ST",class(manifold)))) {         # If we are NOT in a space-time scenario
 
             G <- auto_basis(manifold =manifold,      # Automatically generate basis functions using
-                            data=data[[d]],          # data with largest spatal extent
+                            data=data[[d]],          # data with largest spatial extent
                             ...,
                             max_basis = max_sp_basis) # max. number of basis functions
 
@@ -140,36 +186,45 @@ FRK <- function(f,                     # formula (compulsory)
         G <- basis  # If user has provided basis functions, just use these
     }
 
-    cat("Modelling using",nbasis(G),"basis functions\n")
+    cat("Modelling using",nbasis(G),"basis functions.\n")
     cat("Constructing SRE model...\n")
 
     S <- SRE(f = f,                            # formula
              data = data,                      # list of datasets
              basis = G,                        # basis functions
              BAUs = BAUs,                      # BAUs
-             est_error=est_error,              # estimate measurement error?
+             est_error = est_error,            # estimate measurement error?
              average_in_BAU = average_in_BAU,  # do not average data over BAUs
+             normalise_wts = normalise_wts,
+             sum_variables = sum_variables,  
              fs_model = fs_model,              # fs model (only "ind" for now)
              vgm_model = vgm_model,            # vgm model for error estimation
-             K_type = K_type)                  # "block-exponential" or "unstructured"
+             K_type = K_type,                  # "block-exponential", "unstructured", "precision"
+             response = response, 
+             link = link, 
+             fs_by_spatial_BAU = fs_by_spatial_BAU)                  
 
     ## After constructing SRE model, fit it
     cat("Fitting SRE model...\n")
-    S <- SRE.fit(SRE_model = S,       # SRE model
+    S <- SRE.fit(S,       # SRE model
                  n_EM = n_EM,         # max. no. of EM iterations
                  tol = tol,           # tolerance at which EM is assumed to have converged
-                 method = method,     # method (only "EM" for now)
+                 method = method,     # method ("EM" or "TMB")
                  lambda = lambda,     # regularisation parameter
-                 print_lik=print_lik) # print log-likelihood at each iteration
+                 print_lik = print_lik, # print log-likelihood at each iteration
+                 optimiser = optimiser, 
+                 known_sigma2fs = known_sigma2fs,
+                 taper = taper,
+                 ...) 
 
     ## Return fitted SRE model
-    S
+    return(S)
 }
 
 
 ## The function below checks the arguments for the function FRK. The code is self-explanatory
 ## This is similar, but slightly different to, .check_args1()
-.check_args_wrapper <- function(f,data,basis,BAUs,est_error) {
+.check_args_wrapper <- function(f,data,basis,BAUs,est_error,response) {
     if(!is(f,"formula")) stop("f needs to be a formula.")
     if(!is(data,"list"))
         stop("Please supply a list of Spatial objects.")
@@ -177,7 +232,7 @@ FRK <- function(f,                     # formula (compulsory)
         stop("All data list elements need to be of class Spatial or ST.")
     if(!all(sapply(data,function(x) all.vars(f)[1] %in% names(x@data))))
         stop("All data list elements to have values for the dependent variable.")
-    if(!est_error & !all(sapply(data,function(x) "std" %in% names(x@data))))
+    if(!est_error && response == "gaussian" & !all(sapply(data,function(x) "std" %in% names(x@data))))
         stop("If observational error is not going to be estimated,
              please supply a field 'std' in the data objects.")
     if(!(is.null(BAUs))) {
